from sqlalchemy.orm import relationship
from app import db

class ModeloNer(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    descripcion = db.Column(db.Text)
    ubicacion = db.Column(db.String(256))
    proyectos = relationship('Proyecto', back_populates="modelo_ner")
    #actores = relationship('Actor', back_populates="modelo_ner")
    actores = relationship("Actor")

    def __repr__(self):
        return f'<Modelo NER {self.nombre}>'

    def save(self):
        if not self.id:
            if not self.ubicacion:
                self.ubicacion = self.id
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return ModeloNer.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return ModeloNer.query.get(id)

class Proyecto(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    descripcion = db.Column(db.Text)
    distancia_palabras = db.Column(db.Integer)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id', ondelete='CASCADE'), nullable=False)
    modelo_ner_id = db.Column(db.Integer, db.ForeignKey('modelo_ner.id'), nullable=False)
    modelo_ner = relationship("ModeloNer", back_populates="proyectos")
    documentos = db.relationship("Documento", backref="proyecto", lazy='dynamic', cascade="all, delete-orphan")

    def __repr__(self):
        return f'<Proyecto {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return Proyecto.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return Proyecto.query.get(id)


class Documento(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    ubicacion = db.Column(db.String(256))
    proyecto_id = db.Column(db.Integer, db.ForeignKey('proyecto.id', ondelete='CASCADE'), nullable=False)


    def __repr__(self):
        return f'<Documento {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return Documento.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return Documento.query.get(id)

class Nivel(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    actores = relationship("Actor")

    def __repr__(self):
        return f'<Nivel {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return Nivel.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return Nivel.query.get(id)

class Sector(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    actores = relationship("Actor")

    def __repr__(self):
        return f'<Sector {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return Sector.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return Sector.query.get(id)

class TipoActor(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    actores = relationship("Actor")

    def __repr__(self):
        return f'<TipoActor {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return TipoActor.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return TipoActor.query.get(id)

class Actor(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    nivel_id = db.Column(db.Integer, db.ForeignKey('nivel.id'), nullable=False)
    sector_id = db.Column(db.Integer, db.ForeignKey('sector.id'), nullable=False)
    tipo_actor_id = db.Column(db.Integer, db.ForeignKey('tipo_actor.id'), nullable=False)
    modelo_ner_id = db.Column(db.Integer, db.ForeignKey('modelo_ner.id'), nullable=False)
    modelo_ner = relationship("ModeloNer", back_populates="actores")
    sinonimos = relationship("SinonimoActor")

    def __repr__(self):
        return f'<Proyecto {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return Actor.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return Actor.query.get(id)

class SinonimoActor(db.Model):
    id = db.Column(db.Integer, primary_key=True)
    nombre = db.Column(db.String(256), nullable=False)
    actor_id = db.Column(db.Integer, db.ForeignKey('actor.id'), nullable=False)

    def __repr__(self):
        return f'<Sinónimo Actor {self.nombre}>'

    def save(self):
        if not self.id:
            db.session.add(self)
        db.session.commit()

    @staticmethod
    def get_all():
        return SinonimoActor.query.all()
    
    def delete(self):
        db.session.delete(self)
        db.session.commit()

    @staticmethod
    def get_by_id(id):
        return SinonimoActor.query.get(id)